<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

use CleverReach\WooCommerce\Components\Util\HTTP_Helper;

/**
 * Class Clever_Reach_Base_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Base_Controller {
	/**
	 * Is request call internal.
	 *
	 * @var bool
	 */
	protected $is_internal = true;

	/**
	 * Processes request. Reads 'action' parameter and calls action method if provided.
	 *
	 * @param string $action Request action.
	 */
	public function process( $action = '' ) {
		if ( $this->is_internal ) {
			$this->validate_internal_call();
		}

		if ( empty( $action ) ) {
			$action = HTTP_Helper::get_param( 'action' );
		}

		if ( $action ) {
			if ( method_exists( $this, $action ) ) {
				$this->$action();
			} else {
				$this->return_json( array( 'error' => "Method $action does not exist!" ), 404 );
			}
		}
	}

	/**
	 * Validates if call made from plugin code is secure by checking session token.
	 * If call is not secure, returns 401 status and terminates request.
	 */
	protected function validate_internal_call() {
		$logged_user_id = get_current_user_id();

		// In some cases calls are being made from cleverreach.
		if ( empty( $logged_user_id ) && isset( $_SERVER['HTTP_REFERER'] ) &&
			 'https://rest.cleverreach.com/' !== $_SERVER['HTTP_REFERER'] ) {
			status_header( 401 );
			nocache_headers();

			exit();
		}
	}

	/**
	 * Sets response header content type to json, echos supplied $data as a json string and terminates request.
	 *
	 * @param array $data Array to be returned as a json response.
	 * @param int   $status_code Response status code.
	 */
	protected function return_json( array $data, $status_code = 200 ) {
		wp_send_json( $data, $status_code );
	}

	/**
	 * Sets response header content type to plain/text, echos supplied $data as a json string and terminates request.
	 *
	 * @param string $data Array to be returned as a json response.
	 * @param int    $status_code Response status code.
	 */
	protected function return_plain_text( $data, $status_code = 200 ) {
		status_header( $status_code );
		echo sanitize_text_field( $data );
		die();
	}

	/**
	 * Returns 404 response and terminates request.
	 */
	protected function redirect404() {
		status_header( 404 );
		nocache_headers();

		require get_404_template();

		exit();
	}

	/**
	 * Dies with defined status code.
	 *
	 * @param int $status response status code.
	 */
	protected function die_with_status( $status ) {
		status_header( $status );
		die();
	}
}
