<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use CleverReach\WooCommerce\Components\Exceptions\Invalid_Filter_Settings_Exception;
use CleverReach\WooCommerce\Components\Exceptions\Invalid_Sync_Settings_Exception;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Automation\Site_Automation_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Order_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Buyer_Sync_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Contact_Sync_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Subscriber_Sync_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Sync_Settings_Service;
use CleverReach\WooCommerce\Components\Util\HTTP_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\BlacklistFilter\Contracts\BlacklistFilterService;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\BlacklistFilter\DTO\BlacklistFilterConfig;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\BlacklistFilter\Exceptions\StaticFilterNotValidException;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\BlacklistFilter\Exceptions\WildcardFilterNotValidException;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Order\Contracts\OrderService as Base_Order_Service;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\SyncSettings\Contracts\SyncSettingsService as Base_Sync_Settings_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

/**
 * Class Clever_Reach_Sync_Settings_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Sync_Settings_Controller extends Clever_Reach_Base_Controller {

	/**
	 * Sync settings service.
	 *
	 * @var Sync_Settings_Service $sync_service
	 */
	private $sync_settings_service;

	/**
	 * @var BlacklistFilterService
	 */
	private $blacklist_filter_service;

	/**
	 * Saves sync settings.
	 */
	public function save_sync_settings() {
		try {
			$this->save_filter_configuration();
			$this->save_services();
			$this->return_json( array( 'success' => true ) );
		} catch ( StaticFilterNotValidException $e ) {
			$this->return_json( array( 'success' => false, 'message' => __( 'Static filter is not valid. Please enter up to 500 coma-separated emails.', 'cleverreach-wc' ) ) );
		} catch ( WildcardFilterNotValidException $e ) {
			$this->return_json( array( 'success' => false, 'message' => __( 'Wildcard filter pattern is not valid. Please use * for any sequence of characters and ? for any single character.', 'cleverreach-wc' ) ) );
		} catch ( Invalid_Sync_Settings_Exception $e ) {
			$this->return_json(
				array(
					'success' => false,
					'message' => $e->getMessage(),
				)
			);
		}
	}

	/**
	 * Retrieves sync settings
	 */
	public function get_sync_settings() {
		$enabled_services        = $this->get_enabled_services_formatted();
		$site_automation_service = new Site_Automation_Service();
		$cart_automation         = $site_automation_service->get();
		$filter_configuration    = $this->get_filter_service()->getBlacklistFilterConfig();

		$this->return_json(
			array(
				'enabled_services'     => $enabled_services,
				'thea_activated'       => null !== $cart_automation && 'created' === $cart_automation->getStatus(),
				'filter_configuration' => null !== $filter_configuration ? $filter_configuration->toArray() : null
			)
		);
	}

	/**
	 * Save filter configuration
	 *
	 * @return void
	 * @throws Invalid_Filter_Settings_Exception
	 */
	private function save_filter_configuration() {
		$this->validate_filter_payload();
		$settings     = json_decode( HTTP_Helper::get_param( 'filterSettings' ), true );
		$filterConfig = new BlacklistFilterConfig( $settings['filterType'], $settings['input'] );
		$this->get_filter_service()->saveBlacklistFilterConfig( $filterConfig );
	}

	/**
	 * Validate filter payload
	 *
	 * @return void
	 */
	private function validate_filter_payload() {

		$settings = json_decode( HTTP_Helper::get_param( 'filterSettings' ), true );
		if ( empty( $settings ) || ! array_key_exists( 'filterType', $settings ) || ! array_key_exists( 'input', $settings ) ) {
			throw new Invalid_Filter_Settings_Eception( 'Invalid filter configuration' );
		}
	}

	/**
	 * Saves services
	 *
	 * @throws Invalid_Sync_Settings_Exception Exception if invalid sync settings.
	 */
	private function save_services() {
		$settings      = HTTP_Helper::get_param( 'syncSettings' );
		$sync_settings = explode( ',', $settings );

		$this->validate( $sync_settings );

		$enabled_services = array(
			ServiceRegister::getService( Subscriber_Sync_Service::CLASS_NAME ),
		);

		if ( in_array( 'buyers', $sync_settings, true ) ) {
			$enabled_services[] = ServiceRegister::getService( Buyer_Sync_Service::CLASS_NAME );
		}

		if ( in_array( 'contacts', $sync_settings, true ) ) {
			$enabled_services[] = ServiceRegister::getService( Contact_Sync_Service::CLASS_NAME );
		}

		$this->get_sync_settings_service()->setEnabledServices( $enabled_services );
	}

	/**
	 * Validates sync settings params
	 *
	 * @param array $sync_settings Array of sync settings.
	 *
	 * @throws Invalid_Sync_Settings_Exception Exception if invalid sync settings.
	 */
	private function validate( $sync_settings ) {
		if ( ! in_array( 'subscribers', $sync_settings, true ) ) {
			throw new Invalid_Sync_Settings_Exception( __( 'Subscribers must be selected.', 'cleverreach-wc' ) );
		}

		if ( in_array( 'contacts', $sync_settings, true ) &&
		     ! in_array( 'buyers', $sync_settings, true ) ) {
			throw new Invalid_Sync_Settings_Exception(
				__(
					'Contacts cannot be selected without buyers.',
					'cleverreach-wc'
				)
			);
		}
	}

	/**
	 * Returns enabled services
	 *
	 * @return array
	 */
	private function get_enabled_services_formatted() {
		$buyer            = false;
		$contact          = false;
		$enabled_services = $this->get_sync_settings_service()->getEnabledServices();

		foreach ( $enabled_services as $service ) {
			if ( $service->getUuid() === 'buyer-service' ) {
				$buyer = true;
			}

			if ( $service->getUuid() === 'contact-service' ) {
				$contact = true;
			}
		}

		return array(
			'buyers'   => $buyer,
			'contacts' => $contact,
		);
	}

	/**
	 * Retrieves Sync settings service.
	 *
	 * @return Sync_Settings_Service
	 */
	private function get_sync_settings_service() {
		if ( null === $this->sync_settings_service ) {
			$this->sync_settings_service = ServiceRegister::getService( Base_Sync_Settings_Service::CLASS_NAME );
		}

		return $this->sync_settings_service;
	}

	/**
	 * Retrieves blacklist filter service
	 *
	 * @return BlacklistFilterService
	 */
	private function get_filter_service() {
		if ( null === $this->blacklist_filter_service ) {
			$this->blacklist_filter_service = ServiceRegister::getService( BlacklistFilterService::CLASS_NAME );
		}

		return $this->blacklist_filter_service;
	}

	/**
	 * Retrieves order service.
	 *
	 * @return Order_Service
	 */
	protected function get_order_service() {
		/** @noinspection PhpIncompatibleReturnTypeInspection */
		return ServiceRegister::getService( Base_Order_Service::CLASS_NAME );
	}
}
